//*** TDLPortIO: DriverLINX Port IO Driver wrapper component DEMO ************
//**                                                                        **
//** File: Main.pas                                                         **
//**                                                                        **
//** Copyright (c) 1999 John Pappas (DiskDude). All rights reserved.        **
//**     This software is CardWare; if you use it, you must send me a       **
//**     PostCard - see the documentation, or contact me for more           **
//**     information.                                                       **
//**                                                                        **
//**     Please notify me if you make any changes to this file.             **
//**     Email: diskdude@poboxes.com                                        **
//**                                                                        **
//*** http://diskdude.cjb.net/ ***********************************************

unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  PortIO, StdCtrls, ExtCtrls;

//---------------------------------------------------------------------------
// Types
//---------------------------------------------------------------------------

type
   TBase = (bDecimal, bHex);
   TDataType = (dtByte, dtWord, dtDWord);

  
//---------------------------------------------------------------------------
// TMain_Win class
//---------------------------------------------------------------------------

type
  TMain_Win = class(TForm)
    QuitButton: TButton;
    PortGroup: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    PortEdit: TEdit;
    ReadButton: TButton;
    WriteButton: TButton;
    DataEdit: TEdit;
    BaseGroup: TGroupBox;
    Base10Radio: TRadioButton;
    Base16Radio: TRadioButton;
    TypeGroup: TGroupBox;
    ByteRadio: TRadioButton;
    WordRadio: TRadioButton;
    DWordRadio: TRadioButton;
    AboutButton: TButton;
    DLPortIO: TDLPortIO;
    CheckBox1: TCheckBox;
    CheckBox2: TCheckBox;
    CheckBox3: TCheckBox;
    CheckBox4: TCheckBox;
    CheckBox5: TCheckBox;
    CheckBox6: TCheckBox;
    CheckBox7: TCheckBox;
    CheckBox8: TCheckBox;
    Timer1: TTimer;
    Timer2: TTimer;
    procedure AboutButtonClick(Sender: TObject);
    procedure Base10RadioClick(Sender: TObject);
    procedure Base16RadioClick(Sender: TObject);
    procedure QuitButtonClick(Sender: TObject);
    procedure ReadButtonClick(Sender: TObject);
    procedure WriteButtonClick(Sender: TObject);
    procedure ByteRadioClick(Sender: TObject);
    procedure WordRadioClick(Sender: TObject);
    procedure DWordRadioClick(Sender: TObject);
    procedure CheckBox1Click(Sender: TObject);
    procedure CheckBox2Click(Sender: TObject);
    procedure CheckBox3Click(Sender: TObject);
    procedure CheckBox4Click(Sender: TObject);
    procedure CheckBox5Click(Sender: TObject);
    procedure CheckBox6Click(Sender: TObject);
    procedure CheckBox7Click(Sender: TObject);
    procedure CheckBox8Click(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure Timer2Timer(Sender: TObject);
  private
    { Private declarations }
    FBaseMode : TBase;
    FDataType : TDataType;

  public
    { Public declarations }
    constructor Create(Owner : TComponent); override;
    procedure refresh;
  end;

var
  Main_Win: TMain_Win;

implementation


const
   // Masks
   BIT0 : Byte = $01;
   BIT1 : Byte = $02;
   BIT2 : Byte = $04;
   BIT3 : Byte = $08;
   BIT4 : Byte = $10;
   BIT5 : Byte = $20;
   BIT6 : Byte = $40;
   BIT7 : Byte = $80;

{$R *.DFM}

//---------------------------------------------------------------------------
// Constructor for TMain_Win
//---------------------------------------------------------------------------
constructor TMain_Win.Create(Owner : TComponent);
begin
   inherited Create(Owner);

   FBaseMode:=bHex;   // Hex mode as default
   FDataType:=dtByte; // Byte mode as default

   // Default port
   PortEdit.Text:='$378'; // LPT1:
   DataEdit.Text:='$00';

   // Driver is in the same directory as the demo.exe file!
   DLPortIO.DriverPath:=ExtractFileDir(ParamStr(0));
   // Open the DriverLINX driver
   DLPortIO.OpenDriver();

   if (not DLPortIO.ActiveHW) then
   begin
      MessageDlg('Could not open the DriverLINX driver.', mtError, [mbOK], 0);
      PortGroup.Enabled:=false;
      BaseGroup.Enabled:=false;
      TypeGroup.Enabled:=false;
   end;
end;

//---------------------------------------------------------------------------
// Base10RadioClick()
//    Change the base of the edit boxes to decimal
//---------------------------------------------------------------------------
procedure TMain_Win.Base10RadioClick(Sender: TObject);
begin
   if (FBaseMode<>bDecimal) then
   begin
      // Now in decimal mode
      FBaseMode:=bDecimal;
      try
         PortEdit.Text:=IntToStr(StrToInt(PortEdit.Text));
      except
         PortEdit.Text:='0';
      end;

      try
         DataEdit.Text:=IntToStr(StrToInt(DataEdit.Text));
      except
         DataEdit.Text:='0';
      end;
   end;
end;

//---------------------------------------------------------------------------
// Base16RadioClick()
//    Change the base of the edit boxes to hex
//---------------------------------------------------------------------------
procedure TMain_Win.Base16RadioClick(Sender: TObject);
begin
   if (FBaseMode<>bHex) then
   begin
      // Now in hex mode
      FBaseMode:=bHex;
      try
         PortEdit.Text:='$'+IntToHex(StrToInt(PortEdit.Text),0);
      except
         DataEdit.Text:='$0000';
      end;

      try
         DataEdit.Text:='$'+IntToHex(StrToInt(DataEdit.Text),0);
      except
         DataEdit.Text:='$00';
      end;
   end;
end;

//---------------------------------------------------------------------------
// QuitButtonClick()
//    Quit the demo
//---------------------------------------------------------------------------
procedure TMain_Win.QuitButtonClick(Sender: TObject);
begin
   Close;
end;

//---------------------------------------------------------------------------
// ReadButtonClick()
//    Reads a byte into the DataEdit box
//---------------------------------------------------------------------------
procedure TMain_Win.ReadButtonClick(Sender: TObject);
var
   DataPort : Word;     // Port to read data from
   DataRead : Longword; // Data read from port
begin
   // Get the data port address
   try
      DataPort:=Word(StrToInt(PortEdit.Text));
   except
      MessageDlg('You have specified an invalid port.'+#13+
                 'No action performed.',
                 mtError, [mbOK], 0);
      Exit;
   end;

   // Read the data
   case FDataType of
      dtByte:  DataRead:=DLPortIO.Port[DataPort];
      dtWord:  DataRead:=DLPortIO.PortW[DataPort];
      dtDWord: DataRead:=DLPortIO.PortL[DataPort];
   end;

   if (FBaseMode=bDecimal) then
      DataEdit.Text:=IntToStr(DataRead)
   else
      DataEdit.Text:='$'+IntToHex(DataRead, 0);
end;

//---------------------------------------------------------------------------
// WriteButtonClick()
//    Writes a byte of data from DataEdit to PortEdit
//---------------------------------------------------------------------------
procedure TMain_Win.WriteButtonClick(Sender: TObject);
var
   DataPort  : Word;     // Port to read data from
   DataWrite : Longword; // Data to write to port
begin
   // Get the data port address
   try
      DataPort:=Word(StrToInt(PortEdit.Text));
   except
      MessageDlg('You have specified an invalid port.'+#13+
                 'No action performed.',
                 mtError, [mbOK], 0);
      Exit;
   end;

   // Get the data to write
   try
      DataWrite:=Longword(StrToInt(DataEdit.Text));
   except
      MessageDlg('You have specified an invalid port.'+#13+
                 'No action performed.',
                 mtError, [mbOK], 0);
      Exit;
   end;

   // Write the data
   case FDataType of
      dtByte:  DLPortIO.Port[DataPort]:=Byte(DataWrite and $FF);
      dtWord:  DLPortIO.PortW[DataPort]:=Word(DataWrite and $FFFF);
      dtDWord: DLPortIO.PortL[DataPort]:=DataWrite;
   end;
end;

//---------------------------------------------------------------------------
// ByteRadioClick()
//    Set for Byte writing
//---------------------------------------------------------------------------
procedure TMain_Win.ByteRadioClick(Sender: TObject);
begin
   FDataType:=dtByte;
end;

//---------------------------------------------------------------------------
// WordRadioClick()
//    Set for Word writing
//---------------------------------------------------------------------------
procedure TMain_Win.WordRadioClick(Sender: TObject);
begin
   FDataType:=dtWord;
end;

//---------------------------------------------------------------------------
// DWordRadioClick()
//    Set for Double Word writing
//---------------------------------------------------------------------------
procedure TMain_Win.DWordRadioClick(Sender: TObject);
begin
   FDataType:=dtDWord;
end;

//---------------------------------------------------------------------------
// AboutButtonClick()
//    Shows the about dialog box
//---------------------------------------------------------------------------
procedure TMain_Win.AboutButtonClick(Sender: TObject);
begin
   MessageDlg(
      'DriverLINX driver wrapper component Demo'+#13+
      'Copyright (c) 1999 John Pappas (DiskDude). All rights reserved.'+#13+#13+
      'See http://diskdude.cjb.net/ for the latest version.',
      mtInformation,
      [mbOK],
      0);
end;

procedure TMain_Win.CheckBox1Click(Sender: TObject);
begin
  if CheckBox1.Checked then  DLPortIO.Port[$378] := DLPortIO.Port[$378] or 1
  else DLPortIO.Port[$378]:=DLPortIO.Port[$378] and not 1;

end;

procedure TMain_Win.CheckBox2Click(Sender: TObject);
begin
  if CheckBox2.Checked then  DLPortIO.Port[$378] := DLPortIO.Port[$378] or 2
  else DLPortIO.Port[$378]:=DLPortIO.Port[$378] and not 2;
end;

procedure TMain_Win.CheckBox3Click(Sender: TObject);
begin
  if CheckBox3.Checked then  DLPortIO.Port[$378] := DLPortIO.Port[$378] or 4
  else DLPortIO.Port[$378]:=DLPortIO.Port[$378] and not 4;
end;

procedure TMain_Win.CheckBox4Click(Sender: TObject);
begin
  if CheckBox4.Checked then  DLPortIO.Port[$378] := DLPortIO.Port[$378] or 8
  else DLPortIO.Port[$378]:=DLPortIO.Port[$378] and not 8;
end;

procedure TMain_Win.CheckBox5Click(Sender: TObject);
begin
  if CheckBox5.Checked then  DLPortIO.Port[$378] := DLPortIO.Port[$378] or 16
  else DLPortIO.Port[$378]:=DLPortIO.Port[$378] and not 16;
end;

procedure TMain_Win.CheckBox6Click(Sender: TObject);
begin
  if CheckBox6.Checked then  DLPortIO.Port[$378] := DLPortIO.Port[$378] or 32
  else DLPortIO.Port[$378]:=DLPortIO.Port[$378] and not 32;
end;

procedure TMain_Win.CheckBox7Click(Sender: TObject);
begin
  if CheckBox7.Checked then  DLPortIO.Port[$378]:=DLPortIO.Port[$378] or 64
  else  DLPortIO.Port[$378]:=DLPortIO.Port[$378] and not 64;
end;

procedure TMain_Win.CheckBox8Click(Sender: TObject);
begin
  if CheckBox8.Checked then  DLPortIO.Port[$378] := DLPortIO.Port[$378] or 128
  else DLPortIO.Port[$378]:=DLPortIO.Port[$378] and not 128;
end;

procedure TMain_Win.refresh;
begin
  CheckBox1.Checked := (DlPortIO.Port[$378] and BIT0)<>0;
  CheckBox2.Checked := (DlPortIO.Port[$378] and BIT1)<>0;
  CheckBox3.Checked := (DlPortIO.Port[$378] and BIT2)<>0;
  CheckBox4.Checked := (DlPortIO.Port[$378] and BIT3)<>0;
  CheckBox5.Checked := (DlPortIO.Port[$378] and BIT4)<>0;
  CheckBox6.Checked := (DlPortIO.Port[$378] and BIT5)<>0;
  CheckBox7.Checked := (DlPortIO.Port[$378] and BIT6)<>0;
  CheckBox8.Checked := (DlPortIO.Port[$378] and BIT7)<>0;
end;

procedure TMain_Win.Timer1Timer(Sender: TObject);
begin
refresh;
end;

procedure TMain_Win.Timer2Timer(Sender: TObject);
begin
  CheckBox5.Checked := not CheckBox5.Checked;
end;

end.
